@@ -1,6 +1,6 @@
 /* b-esds.h --- embarrassingly simple data structures
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
 
    This file is part of GNU RCS.
 
@@ -30,9 +30,20 @@ struct wlink
   struct wlink *next;
 };
 
-extern struct link *extend (struct link *tp, void const *x, struct divvy *to);
-extern struct wlink *wextend (struct wlink *tp, void *x, struct divvy *to);
-extern struct link *prepend (void const *x, struct link *ls, struct divvy *to);
-extern struct wlink *wprepend (void *x, struct wlink *ls, struct divvy *to);
+extern struct link *
+extend (struct link *tp, void const *x, struct divvy *to)
+  ALL_NONNULL;
+
+extern struct wlink *
+wextend (struct wlink *tp, void *x, struct divvy *to)
+  ALL_NONNULL;
+
+extern struct link *
+prepend (void const *x, struct link *ls, struct divvy *to)
+  ALL_NONNULL;
+
+extern struct wlink *
+wprepend (void *x, struct wlink *ls, struct divvy *to)
+  ALL_NONNULL;
 
 /* b-esds.h ends here */
