@@ -1,6 +1,6 @@
 /* b-excwho.c --- exclusivity / identity
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -157,6 +157,14 @@ setrid (void)
 #endif
 }
 
+#if USER_OVER_LOGNAME
+#define CONSULT_FIRST   "USER"
+#define CONSULT_SECOND  "LOGNAME"
+#else
+#define CONSULT_FIRST   "LOGNAME"
+#define CONSULT_SECOND  "USER"
+#endif
+
 char const *
 getusername (bool suspicious)
 /* Get and return the caller's login name.
@@ -169,8 +177,8 @@ getusername (bool suspicious)
 
       /* Prefer ‘getenv’ unless ‘suspicious’; it's much faster.  */
       if (suspicious
-          || (!JAM (cgetenv ("LOGNAME"))
-              && !JAM (cgetenv ("USER"))
+          || (!JAM (cgetenv (CONSULT_FIRST))
+              && !JAM (cgetenv (CONSULT_SECOND))
               && !(0 == getlogin_r (buf, BUFSIZ)
                    && JAM (str_save (buf)))))
         {
@@ -211,10 +219,10 @@ caller_login_p (char const *login)
 }
 
 struct link *
-lock_memq (struct link *ls, bool loginp, void const *x)
+lock_memq (struct link *ls, bool login, void const *x)
 /* Search ‘ls’, which should be initialized by caller to have its ‘.next’
    pointing to ‘GROK (locks)’, for a lock that matches ‘x’ and return the
-   link whose cadr is the match, else NULL.  If ‘loginp’, ‘x’ is a login
+   link whose cadr is the match, else NULL.  If ‘login’, ‘x’ is a login
    (string), else it is a delta.  */
 {
   struct rcslock const *rl;
@@ -222,7 +230,7 @@ lock_memq (struct link *ls, bool loginp, void const *x)
   for (; ls->next; ls = ls->next)
     {
       rl = ls->next->entry;
-      if (loginp
+      if (login
           ? STR_SAME (x, rl->login)
           : x == rl->delta)
         return ls;
