@@ -1,6 +1,6 @@
 /* b-excwho.h --- exclusivity / identity
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -20,18 +20,24 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-extern bool stat_mine_p (struct stat *st);
+extern bool stat_mine_p (struct stat *st)
+  ALL_NONNULL;
 extern bool currently_setuid_p (void);
 extern void nosetid (void);
 extern void seteid (void);
 extern void setrid (void);
 extern char const *getusername (bool suspicious);
 extern char const *getcaller (void);
-extern bool caller_login_p (char const *login);
-extern struct link *lock_memq (struct link *ls, bool loginp, void const *x);
-extern struct rcslock const *lock_on (struct delta const *delta);
-extern void lock_drop (struct link *box, struct link *tp);
-extern int addlock_maybe (struct delta *delta, bool selfsame, bool verbose);
+extern bool caller_login_p (char const *login)
+  ALL_NONNULL;
+extern struct link *lock_memq (struct link *ls, bool login, void const *x)
+  ALL_NONNULL;
+extern struct rcslock const *lock_on (struct delta const *delta)
+  ALL_NONNULL;
+extern void lock_drop (struct link *box, struct link *tp)
+  ALL_NONNULL;
+extern int addlock_maybe (struct delta *delta, bool selfsame, bool verbose)
+  ALL_NONNULL;
 
 /* Idioms.  */
 
