@@ -1,6 +1,6 @@
 /* b-fb.h --- basic file operations
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -21,17 +21,29 @@
 */
 
 extern int change_mode (int fd, mode_t mode);
-extern void Ierror (void) exiting;
-extern void testIerror (FILE *f);
-extern void Oerror (void) exiting;
-extern void testOerror (FILE *o);
-extern FILE *fopen_safer (char const *filename, char const *type);
-extern void Ozclose (FILE **p);
-extern void aflush (FILE *f);
+exiting
+extern void Ierror (void);
+extern void testIerror (FILE *f)
+  ALL_NONNULL;
+exiting
+extern void Oerror (void);
+extern void testOerror (FILE *o)
+  ALL_NONNULL;
+extern FILE *fopen_safer (char const *filename, char const *type)
+  ALL_NONNULL;
+extern void Ozclose (FILE **p)
+  ALL_NONNULL;
+extern void aflush (FILE *f)
+  ALL_NONNULL;
 extern void oflush (void);
-extern void afputc (int c, FILE *f);
-extern void aputs (char const *s, FILE *iop);
+extern void afputc (int c, FILE *f)
+  ALL_NONNULL;
+extern void newline (FILE *f)
+  ALL_NONNULL;
+extern void aputs (char const *s, FILE *iop)
+  ALL_NONNULL;
 extern void aprintf (FILE *iop, char const *fmt, ...)
+  ARG_NONNULL ((1, 2))
   printf_string (2, 3);
 
 /* b-fb.h ends here */
