@@ -1,6 +1,6 @@
 /* b-feph.h --- (possibly) temporary files
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -23,7 +23,8 @@
 extern void init_ephemstuff (void);
 extern char const *maketemp (int n);
 extern char const *makedirtemp (bool isworkfile);
-extern void keepdirtemp (char const *name);
+extern void keepdirtemp (char const *name)
+  ALL_NONNULL;
 extern void tempunlink (void);
 extern void dirtempunlink (void);
 
