@@ -1,6 +1,6 @@
 /* b-grok.c --- comma-v parsing
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
 
    This file is part of GNU RCS.
 
@@ -64,7 +64,7 @@ struct grok
   struct cbuf bor_no;                   /* branch or revision */
 };
 
-#define STRUCTALLOC(to,type)  alloc (to, #type, sizeof (type))
+#define STRUCTALLOC(to,type)  alloc (to, sizeof (type))
 
 #if CONTEXTUAL
 static void
@@ -94,6 +94,7 @@ pop_context (struct grok *g)
 
 #endif
 
+exiting
 static void
 ignoble (struct grok *g, char const *fmt, ...)
 {
@@ -201,7 +202,7 @@ accb (struct grok *g)
 }
 
 static bool
-maybe_read_num (struct grok *g, bool must_be_delta_p)
+maybe_read_num (struct grok *g, bool must_be_delta)
 /* Return true and save to ‘to’ if a number was read,
    else false.  */
 {
@@ -212,7 +213,7 @@ maybe_read_num (struct grok *g, bool must_be_delta_p)
   skip_whitespace (g);
   while ('.' == g->c || isdigit (g->c))
     {
-      if (must_be_delta_p)
+      if (must_be_delta)
         dots += ('.' == g->c);
       accb (g);
     }
@@ -230,7 +231,7 @@ maybe_read_num (struct grok *g, bool must_be_delta_p)
           p = finish_string (g->to, &XREP (g).size);
         }
       if (trailing_garbage
-          || (must_be_delta_p
+          || (must_be_delta
               && EVENP (dots)))
         BUMMER ("invalid %s: %s", ks_revno, p);
       XREP (g).string = p;
@@ -287,14 +288,14 @@ must_read_snippet (struct grok *g, char const *role)
 #define MASK_OFFMSB  (1ULL << (sizeof (off_t) * 8 - 1))
 
 static void
-start_atat (struct divvy *to, bool blankp)
+start_atat (struct divvy *to, bool blank)
 {
   struct obstack *o = &to->space;
 
   /* We used to check and ensure ‘off_t’ (8-byte) alignment here,
      but that is now done in ‘make_space’ directly.  */
 
-  if (blankp)
+  if (blank)
     obstack_blank (o, sizeof (struct atat));
 }
 
@@ -335,15 +336,21 @@ atat_ineedexp_few (struct atat *atat, size_t i)
 static bool
 maybe_read_atat (struct grok *g, struct atat **res)
 {
-  struct atat *atat;
+  struct atat *atat = NULL;
   off_t beg;
   size_t lno_start;
-  bool newlinep = false;
+  bool newline = false;
 
 #define POS(adjust)  (fro_tello (g->from) + (adjust))
 
   CBEG ("atat");
   skip_whitespace (g);
+
+  /* Don't bother constructing an empty (length zero) atat
+     if there is nothing to do.  */
+  if (SDELIM != g->c)
+    goto done;
+
   lno_start = g->lno;
   beg = POS (-1);
   start_atat (g->systolic, true);
@@ -357,7 +364,7 @@ maybe_read_atat (struct grok *g, struct atat **res)
         {
           if (WITH_NEEDEXP && KDELIM == g->c)
             needexp = true;
-          else if ((newlinep = ('\n' == g->c)))
+          else if ((newline = ('\n' == g->c)))
             g->lno++;
           MORE (g);
         }
@@ -374,7 +381,7 @@ maybe_read_atat (struct grok *g, struct atat **res)
       atat->needexp_count = 0;
       if ((many = MANYP (atat, count)))
         atat->needexp.bitset =
-          zlloc (g->to, "needexp.bitset",
+          zlloc (g->to,
                  (1 + count / 64) * sizeof (*atat->needexp.bitset));
       else
         atat->needexp.direct = 0;
@@ -394,7 +401,7 @@ maybe_read_atat (struct grok *g, struct atat **res)
 #endif  /* WITH_NEEDEXP */
 
       atat->lno = lno_start;
-      atat->line_count = g->lno - atat->lno + !newlinep;
+      atat->line_count = g->lno - atat->lno + !newline;
       atat->beg = beg;
       atat->from = g->from;
 
@@ -407,15 +414,16 @@ maybe_read_atat (struct grok *g, struct atat **res)
                                   + count * sizeof (off_t)));
     }
 
+ done:
   CEND ();
-  return atat;
+  return BOOLEAN (atat);
 }
 
 static void
 must_read_atat (struct grok *g, struct atat **res, char const *role)
 {
   if (! maybe_read_atat (g, res))
-    BUMMER ("missing %s", role);
+    BUMMER ("missing string after %s", role);
 }
 
 #define MUST_ATAT(g,res,kw)  must_read_atat (g, res, TINYKS (kw))
@@ -455,10 +463,10 @@ struct hash
 static struct hash *
 make_hash_table (struct divvy *to, size_t sz)
 {
-  struct hash *ht = alloc (to, "hash-table", sizeof (struct hash));
+  struct hash *ht = alloc (to, sizeof (struct hash));
 
   ht->sz = sz;
-  ht->a = zlloc (to, "hash-table buckets", sz * sizeof (struct wlink *));
+  ht->a = zlloc (to, sz * sizeof (struct wlink *));
   return ht;
 }
 
@@ -516,7 +524,7 @@ struct fwref
 struct repo *
 empty_repo (struct divvy *to)
 {
-  struct repo *repo = zlloc (to, "repo", sizeof (struct repo));
+  struct repo *repo = zlloc (to, sizeof (struct repo));
 
   repo->strict = STRICT_LOCKING;
   repo->expand = -1;
@@ -606,7 +614,7 @@ full (struct divvy *to, struct fro *f)
   /* ci(1) might call ‘grok_resynch’, after having possibly deleted a lock
      (with a side-effecting ‘setcdr’) so we save a copy of the definitions.
      FIXME: Don't ‘setcdr’!  (Alternatively, remove need to resynch.)  */
-  repo->lockdefs = alloc (to, "locker definition",
+  repo->lockdefs = alloc (to,
                           count * sizeof (struct lockdef));
   for (tp = repo->locks; count--; tp = tp->next)
     {
@@ -786,8 +794,8 @@ full (struct divvy *to, struct fro *f)
         {
           RWARN ("user `%s' holds a lock for %s `%s'",
                  lock->login, ks_ner, lock->revno);
-          ny = zlloc (to, "dummy ny", sizeof (struct notyet));
-          ny->d = zlloc (to, "dummy delta", sizeof (struct delta));
+          ny = zlloc (to, sizeof (struct notyet));
+          ny->d = zlloc (to, sizeof (struct delta));
           ny->revno = ny->d->num = lock->revno;
           puthash (to, ny, repo->ht);
         }
