@@ -1,6 +1,6 @@
 /* b-grok.h --- comma-v parsing
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
 
    This file is part of GNU RCS.
 
@@ -18,8 +18,11 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-extern struct repo *empty_repo (struct divvy *to);
-extern struct repo *grok_all (struct divvy *to, struct fro *f);
-extern void grok_resynch (struct repo *repo);
+extern struct repo *empty_repo (struct divvy *to)
+  ALL_NONNULL;
+extern struct repo *grok_all (struct divvy *to, struct fro *f)
+  ALL_NONNULL;
+extern void grok_resynch (struct repo *repo)
+  ALL_NONNULL;
 
 /* b-grok.h ends here */
