@@ -1,6 +1,6 @@
 /* b-isr.c --- interrupt service routine
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -83,7 +83,7 @@ struct isr_scratch
 
 #define ISR(x)  (scratch->x)
 
-void
+char
 access_page (struct isr_scratch *scratch,
              char const *filename, char const *p)
 {
@@ -92,6 +92,9 @@ access_page (struct isr_scratch *scratch,
   ISR (access_name) = filename;
   t = *p;
   ISR (access_name) = NULL;
+  /* Give the compiler (specifically: "gcc -Wunused-but-set-variable")
+     one less reason to complain.  */
+  return t;
 }
 
 static void
@@ -209,7 +212,7 @@ isr_init (bool *be_quiet)
 #if ISR_STACK_SIZE
   stack_t ss =
     {
-      .ss_sp = alloc (PLEXUS, "sigaltstack", ISR_STACK_SIZE),
+      .ss_sp = alloc (PLEXUS, ISR_STACK_SIZE),
       .ss_size = ISR_STACK_SIZE,
       .ss_flags = 0
     };
