@@ -1,6 +1,6 @@
 /* b-kwxout.c --- keyword expansion on output
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -29,29 +29,23 @@
 #include "b-kwxout.h"
 
 static void
-escape_string (register FILE *out, register char const *s)
-/* Output to ‘out’ the string ‘s’,
-   escaping chars that would break ‘ci -k’.  */
+afilename (bool base, FILE *out)
+/* Output to ‘out’ either the full comma-v filename (‘base’ false)
+   or only its basename (‘base’ true).  In the process, escape
+   chars that would break ‘ci -k’.  */
 {
-  register char c;
+  char const *filename = base
+    ? basefilename (REPO (filename))
+    : getfullRCSname ();
+  char c;
 
-  for (;;)
-    switch ((c = *s++))
+  while ((c = *filename++))
+    switch (c)
       {
-      case 0:
-        return;
-      case '\t':
-        aputs ("\\t", out);
-        break;
-      case '\n':
-        aputs ("\\n", out);
-        break;
-      case ' ':
-        aputs ("\\040", out);
-        break;
-      case KDELIM:
-        aputs ("\\044", out);
-        break;
+      case '\t':   aputs ("\\t",   out); break;
+      case '\n':   aputs ("\\n",   out); break;
+      case ' ':    aputs ("\\040", out); break;
+      case KDELIM: aputs ("\\044", out); break;
       case '\\':
         if (VERSION (5) <= BE (version))
           {
@@ -78,7 +72,7 @@ keyreplace (struct pool_found *marker, struct expctx *ctx)
   int c;
   register size_t cs, cw, ls;
   char const *sp1;
-  char datebuf[datesize + zonelenmax];
+  char datebuf[FULLDATESIZE];
   int RCSv;
   int exp;
   bool include_locker = BE (inclusive_of_Locker_in_Id_val);
@@ -105,15 +99,14 @@ keyreplace (struct pool_found *marker, struct expctx *ctx)
           break;
         case Id:
         case Header:
-          escape_string (out,
-                         marker->i == Id || RCSv < VERSION (4)
-                         ? basefilename (REPO (filename)) : getfullRCSname ());
+          afilename (marker->i == Id || RCSv < VERSION (4), out);
           aprintf (out, " %s %s %s %s",
                    delta->num,
                    date2str (date, datebuf),
                    delta->author,
-                   RCSv == VERSION (3) && delta->lockedby ? "Locked"
-                   : delta->state);
+                   (RCSv == VERSION (3) && delta->lockedby
+                    ? "Locked"
+                    : delta->state));
           if (delta->lockedby)
             {
               if (VERSION (5) <= RCSv)
@@ -132,7 +125,7 @@ keyreplace (struct pool_found *marker, struct expctx *ctx)
           break;
         case Log:
         case RCSfile:
-          escape_string (out, basefilename (REPO (filename)));
+          afilename (true, out);
           break;
         case Name:
           if (delta->name)
@@ -142,7 +135,7 @@ keyreplace (struct pool_found *marker, struct expctx *ctx)
           aputs (delta->num, out);
           break;
         case Source:
-          escape_string (out, getfullRCSname ());
+          afilename (false, out);
           break;
         case State:
           aputs (delta->state, out);
@@ -213,7 +206,7 @@ keyreplace (struct pool_found *marker, struct expctx *ctx)
           ;
 
           /* Copy characters before ‘$Log’ into ‘leader’.  */
-          leader = alloc (SINGLE, "leader", 1 + cs);
+          leader = alloc (SINGLE, 1 + cs);
           cp = leader;
           for (cw = 0; cw < cs; cw++)
             {
@@ -248,7 +241,7 @@ keyreplace (struct pool_found *marker, struct expctx *ctx)
             GETCHAR (c, infile);
           while (c != KDELIM);
         }
-      afputc ('\n', out);
+      newline (out);
       awrite (cp, cs, out);
       sp1 = date2str (date, datebuf);
       if (VERSION (5) <= RCSv)
@@ -271,7 +264,7 @@ keyreplace (struct pool_found *marker, struct expctx *ctx)
           continue;
       for (;;)
         {
-          afputc ('\n', out);
+          newline (out);
           awrite (cp, cw, out);
           if (!ls)
             break;
