@@ -1,6 +1,6 @@
 /* b-kwxout.h --- keyword expansion on output
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
 
    This file is part of GNU RCS.
 
@@ -53,6 +53,7 @@ struct expctx
     }                                           \
   while (0)
 
-extern int expandline (struct expctx *ctx);
+extern int expandline (struct expctx *ctx)
+  ALL_NONNULL;
 
 /* b-kwxout.h ends here */
