@@ -1,6 +1,6 @@
 /* b-peer.h --- finding the ‘execv’able name of a peer program
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
 
    This file is part of GNU RCS.
 
@@ -20,8 +20,10 @@
 
 extern struct symdef peer_super;
 
-const char *one_beyond_last_dir_sep (const char *name);
-char const *find_peer_prog (struct symdef *prog);
+const char *one_beyond_last_dir_sep (const char *name)
+  ALL_NONNULL;
+char const *find_peer_prog (struct symdef *prog)
+  ALL_NONNULL;
 
 /* Idioms.  */
 #define PEER_SUPER()  find_peer_prog (&peer_super)
