@@ -1,6 +1,6 @@
 /* RCS common definitions and data structures
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -43,6 +43,11 @@
 
 #define exiting  _Noreturn
 
+/* Some compilers, notably:
+   - IBM XL C/C++ for AIX, V11.1 (5724-X13), Version: 11.01.0000.0019
+   don't like implicitly considering non-‘NULL’ pointers as ‘true’.  */
+#define BOOLEAN(x)  ((x) ? true : false)
+
 /* GCC attributes  */
 
 #define ARG_NONNULL(which)  _GL_ARG_NONNULL (which)
@@ -112,20 +117,17 @@ enum kwsub
    SLASH        char             Principal filename separator.
    SLASHes      ‘case SLASHes:’  Labels all filename separators.
    ABSFNAME(p)  expression       Is p an absolute filename?
-   X_DEFAULT    string           Default value for -x option.
 */
 #if !WOE
 #define TMPDIR "/tmp"
 #define SLASH '/'
 #define SLASHes '/'
 #define ABSFNAME(p)  (isSLASH ((p)[0]))
-#define X_DEFAULT ",v"
 #else /* WOE */
 #define TMPDIR "\\tmp"
 #define SLASH "'\\'"
 #define SLASHes '\\': case '/': case ':'
 #define ABSFNAME(p)  (isSLASH ((p)[0]) || (p)[0] && (p)[1] == ':')
-#define X_DEFAULT "\\,v"
 #endif
 
 /* Must TZ be set for ‘gmtime_r’ to work?  */
@@ -168,10 +170,6 @@ enum kwsub
 #define STRICT_LOCKING  true
 #endif
 
-/* This is good through AD 9,999,999,999,999,999.  */
-#define yearlength                           16
-/* Size of output of ‘time2date’.  */
-#define datesize                             (yearlength + 16)
 /* Delimiter for keywords.  */
 #define KDELIM                               '$'
 /* Separates keywords from values.  */
@@ -338,8 +336,13 @@ enum markers
 /* This is used by ci and rlog.  */
 #define EMPTYLOG "*** empty log message ***"
 
-/* Maxiumum length of time zone string, e.g. "+12:34:56".  */
-#define zonelenmax  9
+/* Buffer sizes for time/date funcs.
+   The six is for the year, good through AD 999,999.
+   (This was chosen so that ‘FULLDATESIZE’ + 1 = 32.)
+   The second is basically ‘(1+ (length ".MM.DD.HH.MM.SS"))’.
+   9 is max len of time zone string, e.g. "+12:34:56".  */
+#define DATESIZE            (6 + 16)
+#define FULLDATESIZE        (DATESIZE + 9)
 
 struct maybe;
 
@@ -417,7 +420,7 @@ struct behavior
   /* The directory portion of ‘PROGRAM (invoke)’.
      -- find_peer_prog  */
 
-  bool unbufferedp;
+  bool unbuffered;
   /* Although standard error should be unbuffered by default,
      don't rely on it.
      -- unbuffer_standard_error  */
@@ -483,7 +486,6 @@ struct behavior
   char const *pe;
   /* Possible endings, a slash-separated list of filename-end
      fragments to consider for recognizing the name of the RCS file.
-     FIXME: Push defaulting into library.
      -- [ci]main [co]main [rcs]main [rcsclean]main [rcsdiff]main
      -- rcssuffix
      -- [rcsmerge]main [rlog]main  */
@@ -504,8 +506,8 @@ struct behavior
   /* The login id of the program user.
      -- getusername  */
 
-  time_t now;
-  /* Cached time from ‘time’.
+  struct timespec now;
+  /* Cached time from ‘gettime’.
      -- now  */
 
   bool fixed_SIGCHLD;
@@ -672,7 +674,7 @@ struct flow
   /* The result file name.
      -- openfcopy swapeditfiles  */
 
-  bool erroneousp;
+  bool erroneous;
   /* True means some (parsing/merging) error was encountered.
      The program should clean up temporary files and exit.
      -- buildjoin syserror generic_error generic_fatal  */
@@ -721,117 +723,169 @@ extern TINY_DECL (state);
 extern TINY_DECL (strict);
 extern TINY_DECL (symbols);
 extern TINY_DECL (text);
-bool looking_at (struct tinysym const *sym, char const *start);
-int recognize_kwsub (struct cbuf const *x);
-int str2expmode (char const *s);
+bool looking_at (struct tinysym const *sym, char const *start)
+  ALL_NONNULL;
+int recognize_kwsub (struct cbuf const *x)
+  ALL_NONNULL;
+int str2expmode (char const *s)
+  ALL_NONNULL;
 char const *kwsub_string (enum kwsub i);
-bool recognize_keyword (char const *string, struct pool_found *found);
+bool recognize_keyword (char const *string, struct pool_found *found)
+  ALL_NONNULL;
 
 /* merger */
 int merge (bool tostdout, char const *edarg,
-           struct symdef three_manifestations[3]);
+           struct symdef three_manifestations[3])
+  ARG_NONNULL ((3));
 
 /* rcsedit */
 struct editstuff *make_editstuff (void);
-void unmake_editstuff (struct editstuff *es);
-int un_link (char const *s);
+void unmake_editstuff (struct editstuff *es)
+  ALL_NONNULL;
+int un_link (char const *s)
+  ALL_NONNULL;
 void openfcopy (FILE *f);
 void finishedit (struct editstuff *es, struct delta const * delta,
                  FILE *outfile, bool done);
-void snapshotedit (struct editstuff *es, FILE *f);
-void copystring (struct editstuff *es, struct atat *atat);
-void enterstring (struct editstuff *es, struct atat *atat);
+void snapshotedit (struct editstuff *es, FILE *f)
+  ALL_NONNULL;
+void copystring (struct editstuff *es, struct atat *atat)
+  ALL_NONNULL;
+void enterstring (struct editstuff *es, struct atat *atat)
+  ALL_NONNULL;
 void editstring (struct editstuff *es, struct atat const *script,
-                 struct delta const *delta);
-struct fro *rcswriteopen (struct maybe *m);
+                 struct delta const *delta)
+  ARG_NONNULL ((1, 2));
+struct fro *rcswriteopen (struct maybe *m)
+  ALL_NONNULL;
 int chnamemod (FILE **fromp, char const *from, char const *to,
-               int set_mode, mode_t mode, time_t mtime);
-int setmtime (char const *file, time_t mtime);
-int findlock (bool delete, struct delta **target);
-int addsymbol (char const *num, char const *name, bool rebind);
+               int set_mode, mode_t mode, const struct timespec mtime)
+  ALL_NONNULL;
+int setmtime (char const *file, const struct timespec mtime)
+  ALL_NONNULL;
+int findlock (bool delete, struct delta **target)
+  ALL_NONNULL;
+int addsymbol (char const *num, char const *name, bool rebind)
+  ALL_NONNULL;
 bool checkaccesslist (void);
 int dorewrite (bool lockflag, int changed);
-int donerewrite (int changed, time_t newRCStime);
+int donerewrite (int changed, const struct timespec mtime);
 void ORCSclose (void);
 void ORCSerror (void);
-void unexpected_EOF (void)
-  exiting;
-void initdiffcmd (struct diffcmd *dc);
+exiting
+void unexpected_EOF (void);
+void initdiffcmd (struct diffcmd *dc)
+  ALL_NONNULL;
 int getdiffcmd (struct fro *finfile, bool delimiter,
-                FILE *foutfile, struct diffcmd *dc);
+                FILE *foutfile, struct diffcmd *dc)
+  ARG_NONNULL ((1, 4));
 
 /* rcsfcmp */
 int rcsfcmp (struct fro *xfp, struct stat const *xstatp,
-             char const *uname, struct delta const *delta);
+             char const *uname, struct delta const *delta)
+  ALL_NONNULL;
 
 /* rcsfnms */
-char const *basefilename (char const *p);
-char const *rcssuffix (char const *name);
-struct fro *rcsreadopen (struct maybe *m);
+char const *basefilename (char const *p)
+  ALL_NONNULL;
+char const *rcssuffix (char const *name)
+  ALL_NONNULL;
+struct fro *rcsreadopen (struct maybe *m)
+  ALL_NONNULL;
 int pairnames (int argc, char **argv, open_rcsfile_fn *rcsopen,
-               bool mustread, bool quiet);
+               bool mustread, bool quiet)
+  ALL_NONNULL;
 char const *getfullRCSname (void);
 bool isSLASH (int c);
 
 /* rcsgen */
 char const *buildrevision (struct wlink const *deltas,
                            struct delta *target,
-                           FILE *outfile, bool expandflag);
-struct cbuf cleanlogmsg (char const *m, size_t s);
+                           FILE *outfile, bool expandflag)
+  ARG_NONNULL ((1, 2));
+struct cbuf cleanlogmsg (char const *m, size_t s)
+  ALL_NONNULL;
 bool ttystdin (void);
 int getcstdin (void);
 bool yesorno (bool default_answer, char const *question, ...)
+  ARG_NONNULL ((2))
   printf_string (2, 3);
 void write_desc_maybe (FILE *to);
-void putdesc (struct cbuf *cb, bool textflag, char *textfile);
-struct cbuf getsstdin (char const *option, char const *name, char const *note);
-void format_assocs (FILE *out, char const *fmt);
-void format_locks (FILE *out, char const *fmt);
+void putdesc (struct cbuf *cb, bool textflag, char *textfile)
+  ARG_NONNULL ((1));
+struct cbuf getsstdin (char const *option, char const *name, char const *note)
+  ALL_NONNULL;
+void format_assocs (FILE *out, char const *fmt)
+  ALL_NONNULL;
+void format_locks (FILE *out, char const *fmt)
+  ALL_NONNULL;
 void putadmin (void);
-void puttree (struct delta const *root, FILE *fout);
+void puttree (struct delta const *root, FILE *fout)
+  ARG_NONNULL ((2));
 bool putdtext (struct delta const *delta, char const *srcname,
-               FILE *fout, bool diffmt);
-void putstring (FILE *out, bool delim, struct cbuf s, bool log);
+               FILE *fout, bool diffmt)
+  ALL_NONNULL;
+void putstring (FILE *out, struct cbuf s, bool log)
+  ALL_NONNULL;
 void putdftext (struct delta const *delta, struct fro *finfile,
-                FILE *foutfile, bool diffmt);
+                FILE *foutfile, bool diffmt)
+  ALL_NONNULL;
 
 /* rcskeep */
 bool getoldkeys (struct fro *fp);
 
 /* rcsmap */
 extern enum tokens const ctab[];
-char const *checkid (char const *id, int delimiter);
-char const *checksym (char const *sym, int delimiter);
-void checksid (char const *id);
-void checkssym (char const *sym);
+char const *checkid (char const *id, int delimiter)
+  ALL_NONNULL;
+char const *checksym (char const *sym, int delimiter)
+  ALL_NONNULL;
+void checksid (char const *id)
+  ALL_NONNULL;
+void checkssym (char const *sym)
+  ALL_NONNULL;
 
 /* rcsrev */
 int countnumflds (char const *s);
-struct cbuf take (size_t count, char const *ref);
+struct cbuf take (size_t count, char const *ref)
+  ALL_NONNULL;
 int cmpnum (char const *num1, char const *num2);
-int cmpnumfld (char const *num1, char const *num2, int fld);
-int cmpdate (char const *d1, char const *d2);
-int compartial (char const *num1, char const *num2, int length);
+int cmpnumfld (char const *num1, char const *num2, int fld)
+  ALL_NONNULL;
+int cmpdate (char const *d1, char const *d2)
+  ALL_NONNULL;
+int compartial (char const *num1, char const *num2, int length)
+  ALL_NONNULL;
 struct delta *genrevs (char const *revno, char const *date,
                        char const *author, char const *state,
-                       struct wlink **store);
-struct delta *gr_revno (char const *revno, struct wlink **store);
-struct delta *delta_from_ref (char const *ref);
-bool fully_numeric (struct cbuf *ans, char const *source, struct fro *fp);
-char const *namedrev (char const *name, struct delta *delta);
+                       struct wlink **store)
+  ARG_NONNULL ((1));
+struct delta *gr_revno (char const *revno, struct wlink **store)
+  ALL_NONNULL;
+struct delta *delta_from_ref (char const *ref)
+  ALL_NONNULL;
+bool fully_numeric (struct cbuf *ans, char const *source, struct fro *fp)
+  ARG_NONNULL ((1));
+char const *namedrev (char const *name, struct delta *delta)
+  ARG_NONNULL ((2));
 char const *tiprev (void);
 
 /* rcstime */
-void time2date (time_t unixtime, char date[datesize]);
-void str2date (char const *source, char target[datesize]);
-time_t date2time (char const source[datesize]);
-void zone_set (char const *s);
-char const *date2str (char const date[datesize],
-                      char datebuf[datesize + zonelenmax]);
+void time2date (time_t unixtime, char date[DATESIZE])
+  ALL_NONNULL;
+void str2date (char const *source, char target[DATESIZE])
+  ALL_NONNULL;
+time_t date2time (char const source[DATESIZE])
+  ALL_NONNULL;
+void zone_set (char const *s)
+  ALL_NONNULL;
+char const *date2str (char const date[DATESIZE],
+                      char datebuf[FULLDATESIZE])
+  ALL_NONNULL;
 
 /* rcsutil */
-void thank_you_and_goodnight (int const how)
-  exiting;
+exiting
+void thank_you_and_goodnight (int const how);
 /* These are for ‘thank_you_and_goodnight’.  */
 #define TYAG_ORCSERROR     (1 << 3)
 #define TYAG_DIRTMPUNLINK  (1 << 2)
@@ -839,23 +893,39 @@ void thank_you_and_goodnight (int const how)
 #define TYAG_DIFF          (1 << 0)
 #define TYAG_IMMEDIATE           0
 
-void gnurcs_init (struct program const *program);
+void gnurcs_init (struct program const *program)
+  ALL_NONNULL;
 void gnurcs_goodbye (void);
-void bad_option (char const *option);
+void bad_option (char const *option)
+  ALL_NONNULL;
 void redefined (int c);
-struct cbuf minus_p (char const *xrev, char const *rev);
+void chk_set_rev (const char **rev, char *arg)
+  ALL_NONNULL;
+struct cbuf minus_p (char const *xrev, char const *rev)
+  ALL_NONNULL;
 void parse_revpairs (char option, char *arg, void *data,
                      void (*put) (char const *b, char const *e,
-                                  bool sawsep, void *data));
-void set_empty_log_message (struct cbuf *cb);
+                                  bool sawsep, void *data))
+  ALL_NONNULL;
+void set_empty_log_message (struct cbuf *cb)
+  ALL_NONNULL;
 void ffree (void);
-char *str_save (char const *s);
-char *cgetenv (char const *name);
-void awrite (char const *buf, size_t chars, FILE *f);
-int runv (int infd, char const *outname, char const **args);
+char *str_save (char const *s)
+  ALL_NONNULL;
+char *cgetenv (char const *name)
+  ALL_NONNULL;
+void awrite (char const *buf, size_t chars, FILE *f)
+  ALL_NONNULL;
+int runv (int infd, char const *outname, char const **args)
+  ARG_NONNULL ((3));
 int run (int infd, char const *outname, ...);
-void setRCSversion (char const *str);
-int getRCSINIT (int argc, char **argv, char ***newargv);
+void setRCSversion (char const *str)
+  ALL_NONNULL;
+int getRCSINIT (int argc, char **argv, char ***newargv)
+  ALL_NONNULL;
+struct timespec unspecified_timespec (void);
+struct timespec file_mtime (bool enable, struct stat const *st)
+  ALL_NONNULL;
 
 /* Indexes into ‘BE (sff)’.  */
 #define SFFI_LOCKDIR  0
@@ -863,6 +933,12 @@ int getRCSINIT (int argc, char **argv, char ***newargv);
 
 /* Idioms.  */
 
+#define TIME_UNSPECIFIED       ((time_t) -1)
+#define TIME_UNSPECIFIED_P(x)  ((x) == TIME_UNSPECIFIED)
+
+/* This is for ‘ns’ (2nd) arg to ‘make_timespec’.  */
+#define ZERO_NANOSECONDS  0
+
 #define MEMORY_UNLIMITED  -1
 
 #define BOG_DIFF   (TYAG_TEMPUNLINK | TYAG_DIFF)
