@@ -1,6 +1,6 @@
 /* Check in revisions of RCS files from working files.
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -27,6 +27,8 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include "same-inode.h"
+#include "stat-time.h"
+#include "timespec.h"
 #include "ci.help"
 #include "b-complain.h"
 #include "b-divvy.h"
@@ -59,13 +61,14 @@ struct bud                              /* new growth */
   struct wlink br;                      /* branch to be inserted */
   bool keep;
   struct delta *target;
-  char getcurdate_buffer[datesize];
+  char getcurdate_buffer[DATESIZE];
+  struct timespec work_mtime;
 };
 
 static void
 cleanup (int *exitstatus, struct work *work)
 {
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     *exitstatus = exit_failure;
   fro_zclose (&FLOW (from));
   fro_zclose (&work->fro);
@@ -79,7 +82,14 @@ cleanup (int *exitstatus, struct work *work)
 
 #define OK(x)     (x)->string = finish_string (PLEXUS, &((x)->size))
 #define JAM(x,s)  do { ACCF ("%s", s); OK (x); } while (0)
-#define ADD(x,s)  do { ACCF ("%s", (x)->string); JAM (x, s); } while (0)
+
+#define ADD(x,s)  do                            \
+    {                                           \
+      accumulate_nbytes                         \
+        (PLEXUS, (x)->string, (x)->size);       \
+      JAM (x, s);                               \
+    }                                           \
+  while (0)
 
 static void
 incnum (char const *onum, struct cbuf *nnum)
@@ -461,24 +471,26 @@ getcurdate (struct bud *bud)
 /* Return a pointer to the current date.  */
 {
   if (!bud->getcurdate_buffer[0])
-    time2date (BE (now), bud->getcurdate_buffer);
+    time2date (BE (now.tv_sec), bud->getcurdate_buffer);
   return bud->getcurdate_buffer;
 }
 
 static int
-fixwork (mode_t newworkmode, time_t mtime, struct work *work)
+fixwork (mode_t newworkmode, const struct timespec mtime, struct work *work)
 {
   char const *mani_filename = MANI (filename);
   struct stat *st = &work->st;
 
   return
-    1 < st->st_nlink
-    || (newworkmode & S_IWUSR && !stat_mine_p (st))
-    || PROB (setmtime (mani_filename, mtime))
-    ? -1 : st->st_mode == newworkmode ? 0
-    : !PROB (change_mode (work->fro->fd, newworkmode)) ? 0
-    : chmod (mani_filename, newworkmode)
-    ;
+    (1 < st->st_nlink
+     || (newworkmode & S_IWUSR && !stat_mine_p (st))
+     || PROB (setmtime (mani_filename, mtime)))
+    ? -1
+    : (st->st_mode == newworkmode
+       ? 0
+       : (!PROB (change_mode (work->fro->fd, newworkmode))
+          ? 0
+          : chmod (mani_filename, newworkmode)));
 }
 
 static int
@@ -542,7 +554,7 @@ getlogmsg (struct reason *reason, struct bud *bud)
 
   if (bud->keep)
     {
-      char datebuf[datesize + zonelenmax];
+      char datebuf[FULLDATESIZE];
 
       /* Generate standard log message.  */
       date2str (getcurdate (bud), datebuf);
@@ -569,7 +581,7 @@ getlogmsg (struct reason *reason, struct bud *bud)
   if (reason->delayed.size)
     {
       /*Previous log available.  */
-      if (yesorno (true, "reuse log message of previous file? [yn](y): "))
+      if (yesorno (true, "reuse log message of previous file"))
         return reason->delayed;
     }
 
@@ -603,10 +615,10 @@ ci_main (const char *cmd, int argc, char **argv)
 {
   int exitstatus = EXIT_SUCCESS;
   struct reason reason;
-  char altdate[datesize];
-  char olddate[datesize];
-  char newdatebuf[datesize + zonelenmax];
-  char targetdatebuf[datesize + zonelenmax];
+  char altdate[DATESIZE];
+  char olddate[DATESIZE];
+  char newdatebuf[FULLDATESIZE];
+  char targetdatebuf[FULLDATESIZE];
   char *a, **newargv, *textfile;
   char const *author, *krev, *rev, *state;
   char const *diffname, *expname;
@@ -624,7 +636,7 @@ ci_main (const char *cmd, int argc, char **argv)
   bool dolog, newhead;
   bool usestatdate;             /* Use mod time of file for -d.  */
   mode_t newworkmode;           /* mode for working file */
-  time_t mtime, wtime;
+  struct timespec mtime, wtime;
   struct bud bud;
   struct delta *workdelta;
   struct link *symbolic_names = NULL;
@@ -651,7 +663,6 @@ ci_main (const char *cmd, int argc, char **argv)
   Ttimeflag = false;
   altdate[0] = '\0';            /* empty alternate date for -d */
   usestatdate = false;
-  BE (pe) = X_DEFAULT;
 
   argc = getRCSINIT (argc, argv, &newargv);
   argv = newargv;
@@ -669,12 +680,7 @@ ci_main (const char *cmd, int argc, char **argv)
         case 'l':
           keepworkingfile = lockflag = true;
         revno:
-          if (*a)
-            {
-              if (rev)
-                PWARN ("redefinition of %s", ks_revno);
-              rev = a;
-            }
+          chk_set_rev (&rev, a);
           break;
 
         case 'u':
@@ -804,7 +810,7 @@ ci_main (const char *cmd, int argc, char **argv)
   /* (End processing of options.)  */
 
   /* Handle all filenames.  */
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     cleanup (&exitstatus, &work);
   else if (argc < 1)
     PFATAL ("no input file");
@@ -816,6 +822,7 @@ ci_main (const char *cmd, int argc, char **argv)
         char const *mani_filename, *pv;
         struct fro *from;
         struct stat *repo_stat;
+        struct timespec fs_mtime;
         FILE *frew;
         struct delta *tip;
         int kws;
@@ -922,26 +929,29 @@ ci_main (const char *cmd, int argc, char **argv)
         bud.d.name = NULL;
 
         /* Set author.  */
-        if (author)
+        bud.d.author =
           /* Given by ‘-w’.  */
-          bud.d.author = author;
-        else if (bud.keep && (pv = PREV (author)))
-            /* Preserve old author if possible.  */
-          bud.d.author = pv;
-        else
-          /* Otherwise use caller's id.  */
-          bud.d.author = getcaller ();
+          (author
+           ? author
+           /* Preserve old author if possible.  */
+           : (bud.keep && (pv = PREV (author))
+              ? pv
+              /* Otherwise use caller's id.  */
+              : getcaller ()));
 
         /* Set state.  */
-        bud.d.state = default_state;
-        if (state)
+        bud.d.state =
           /* Given by ‘-s’.  */
-          bud.d.state = state;
-        else if (bud.keep && (pv = PREV (state)))
-          /* Preserve old state if possible.  */
-          bud.d.state = pv;
+          (state
+           ? state
+           /* Preserve old state if possible.  */
+           : (bud.keep && (pv = PREV (state))
+              ? pv
+              /* default */
+              : default_state));
 
         /* Compute date.  */
+        bud.work_mtime = get_stat_mtime (&work.st);
         if (usestatdate)
           {
             time2date (work.st.st_mtime, altdate);
@@ -1062,7 +1072,7 @@ ci_main (const char *cmd, int argc, char **argv)
                       while (ftello (frew) < hwm)
                         /* White out any earlier mistake with '\n's.
                            This is unlikely.  */
-                        afputc ('\n', frew);
+                        newline (frew);
                   }
               }
             else
@@ -1110,7 +1120,8 @@ ci_main (const char *cmd, int argc, char **argv)
                 /* Check whether the working file changed during checkin,
                    to avoid producing an inconsistent RCS file.  */
                 if (PROB (fstat (wfd, &checkworkstat))
-                    || work.st.st_mtime != checkworkstat.st_mtime
+                    || 0 != timespec_cmp (get_stat_mtime (&checkworkstat),
+                                          bud.work_mtime)
                     || work.st.st_size != checkworkstat.st_size)
                   {
                     MERR ("file changed during checkin");
@@ -1121,15 +1132,23 @@ ci_main (const char *cmd, int argc, char **argv)
               }
           }
 
-        /* Deduce ‘time_t’ of new revision if it is needed later.  */
-        wtime = (time_t) - 1;
-        if (mtimeflag | Ttimeflag)
-          wtime = date2time (workdelta->date);
+        /* Deduce timestamp of new revision if it is needed later.  */
+        wtime = (mtimeflag | Ttimeflag)
+          ? (usestatdate
+             ? bud.work_mtime
+             : make_timespec (date2time (workdelta->date),
+                              ZERO_NANOSECONDS))
+          : unspecified_timespec ();
+
+        if (Ttimeflag)
+          fs_mtime = file_mtime (from, repo_stat);
 
         if (PROB (donerewrite (changedRCS, !Ttimeflag
-                               ? (time_t) - 1
-                               : ((from && wtime < repo_stat->st_mtime)
-                                  ? repo_stat->st_mtime
+                               ? unspecified_timespec ()
+                               : (PROB (timespec_cmp (wtime, fs_mtime))
+                                  /* File is newer.  */
+                                  ? fs_mtime
+                                  /* Delta is newer.  */
                                   : wtime))))
           continue;
 
@@ -1144,7 +1163,9 @@ ci_main (const char *cmd, int argc, char **argv)
             newworkmode = WORKMODE (repo_stat->st_mode,
                                     !(kws == kwsub_v
                                       || lockthis < BE (strictly_locking)));
-            mtime = mtimeflag ? wtime : (time_t) - 1;
+            mtime = mtimeflag
+              ? wtime
+              : unspecified_timespec ();
 
             /* Expand if it might change or if we can't fix mode, time.  */
             if (changework || PROB (r = fixwork (newworkmode, mtime, &work)))
