@@ -1,6 +1,6 @@
 /* Check out working files from revisions of RCS files.
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include "same-inode.h"
+#include "timespec.h"
 #include "co.help"
 #include "b-complain.h"
 #include "b-divvy.h"
@@ -36,6 +37,10 @@
 #include "b-isr.h"
 #include "b-peer.h"
 
+static const char ks_hws[] = " \t";
+
+#define SKIP_HWS(var)  var += strspn (var, ks_hws)
+
 struct work
 {
   struct stat st;
@@ -67,7 +72,7 @@ cleanup (int *exitstatus, FILE **neworkptr)
 {
   FILE *mstdout = MANI (standard_output);
 
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     *exitstatus = exit_failure;
   fro_zclose (&FLOW (from));
   ORCSclose ();
@@ -93,7 +98,7 @@ rmworkfile (struct work *work)
       char const *mani_filename = MANI (filename);
 
       /* File is writable.  */
-      if (!yesorno (false, "writable %s exists%s; remove it? [ny](n): ",
+      if (!yesorno (false, "writable %s exists%s; remove it",
                     mani_filename, (stat_mine_p (&work->st)
                                     ? ""
                                     : ", and you do not own it")))
@@ -143,45 +148,36 @@ jpush (char const *rev, struct jstuff *js)
 }
 
 static char *
-addjoin (char *joinrev, struct jstuff *js)
-/* Add the number of ‘joinrev’ to ‘js->ls’; return address
-   of char past ‘joinrev’, or NULL if no such revision exists.  */
+addjoin (char *spec, struct jstuff *js)
+/* Tokenize ‘spec’; try to resolve the first token to an
+   existing delta; if found, ‘jpush’ its fully-numeric revno
+   and return the "rest" of ‘spec’ (position after first token).
+   If no delta can be determined, return NULL.  */
 {
-  register char *j;
-  register struct delta *d;
-  char terminator;
+  const char delims[] = " \t\n:,;";
+  char *eot, save;
+  struct delta *cool;
   struct cbuf numrev;
 
-  j = joinrev;
-  for (;;)
-    {
-      switch (*j++)
-        {
-        default:
-          continue;
-        case 0:
-        case ' ':
-        case '\t':
-        case '\n':
-        case ':':
-        case ',':
-        case ';':
-          break;
-        }
-      break;
-    }
-  terminator = *--j;
-  *j = '\0';
-  d = NULL;
-  if (fully_numeric_no_k (&numrev, joinrev))
-    d = delta_from_ref (numrev.string);
-  *j = terminator;
-  if (d)
-    {
-      jpush (d->num, js);
-      return j;
-    }
-  return NULL;
+  /* Locate bounds of token.  */
+  SKIP_HWS (spec);
+  eot = spec + strcspn (spec, delims);
+
+  /* Save the delim, jam a terminating NUL there for the sake of
+     ‘fully_numeric_no_k’, and restore it after the call.  Blech.  */
+  save = *eot;
+  *eot = '\0';
+  cool = (fully_numeric_no_k (&numrev, spec)
+          ? delta_from_ref (numrev.string)
+          : NULL);
+  *eot = save;
+
+  if (! cool)
+    return NULL;
+
+  jpush (cool->num, js);
+  SKIP_HWS (eot);
+  return eot;
 }
 
 static char const *
@@ -224,11 +220,25 @@ getancestor (char const *r1, char const *r2)
 }
 
 static bool
-preparejoin (register char *j, struct jstuff *js)
-/* Parse join list ‘j’ and place pointers to the
-   revision numbers into ‘js->ls’.
-   Set ‘js->lastidx’ to the last index of the list.  */
+preparejoin (register char *argv, struct jstuff *js)
+/* Parse join pairs from ‘argv’; ‘jpush’ their revision numbers.
+   Set ‘js->lastidx’ to the last index of the list.
+   Return ‘true’ if all went well, otherwise ‘false’.  */
 {
+  /* This is a two level parse, w/ whitespace complications:
+     top-level is comma-delimited (straightforward), but
+     join pairs are of the form: HWS REV1 HWS [":" HWS REV2 HWS]
+     where HWS is optional horizontal whitespace (SPC, TAB).
+     Also, the first "pair" can be comprised solely of REV1;
+     in that case, the ":" and REV2 are also optional.
+
+     Most of the HWS skipping is done in subroutine ‘addjoin’.
+
+     TODO: Investigate why on addjoin "failure", it's OK to
+     simply return ‘false’ (and not going through ‘done’).  */
+
+  const char ks_comma[] = ",";
+  char *s, *save, *j;
   bool rv = true;
 
   js->jstuff = make_space ("jstuff");
@@ -241,30 +251,19 @@ preparejoin (register char *j, struct jstuff *js)
     }
 
   js->lastidx = -1;
-  for (;;)
+  for (s = argv;
+       (j = strtok_r (s, ks_comma, &save));
+       s = NULL)
     {
-      while ((*j == ' ') || (*j == '\t') || (*j == ','))
-        j++;
-      if (*j == '\0')
-        break;
       if (!(j = addjoin (j, js)))
         return false;
-      while ((*j == ' ') || (*j == '\t'))
-        j++;
-      if (*j == ':')
+      if (*j++ == ':')
         {
-          j++;
-          while ((*j == ' ') || (*j == '\t'))
-            j++;
-          if (*j != '\0')
-            {
-              if (!(j = addjoin (j, js)))
-                return false;
-            }
-          else
-            {
-              RFATAL ("join pair incomplete");
-            }
+          SKIP_HWS (j);
+          if (*j == '\0')
+            goto incomplete;
+          if (!(j = addjoin (j, js)))
+            return false;
         }
       else
         {
@@ -272,24 +271,21 @@ preparejoin (register char *j, struct jstuff *js)
             {
               char const *two = js->tp->entry;
 
-              /* Derive common ancestor.  */
-              if (! (js->tp->entry = getancestor (js->d->num, two)))
-                {
-                  rv = false;
-                  goto done;
-                }
               /* Common ancestor missing.  */
               jpush (two, js);
+              /* Derive common ancestor.  */
+              if (! (js->tp->entry = getancestor (js->d->num, two)))
+                rv = false;
             }
           else
             {
+            incomplete:
               RFATAL ("join pair incomplete");
             }
         }
     }
   if (js->lastidx < 1)
     RFATAL ("empty join");
- done:
 
   js->ls = pointer_array (PLEXUS, 1 + js->lastidx);
   js->tp = js->head.next;
@@ -383,7 +379,7 @@ buildjoin (char const *initialfile, struct jstuff *js)
   return true;
 
 badmerge:
-  FLOW (erroneousp) = true;
+  FLOW (erroneous) = true;
   return false;
 }
 
@@ -406,7 +402,7 @@ co_main (const char *cmd, int argc, char **argv)
   int expmode, r, workstatstat;
   bool tostdout, Ttimeflag;
   bool selfsame = false;
-  char finaldate[datesize];
+  char finaldate[DATESIZE];
 #if OPEN_O_BINARY
   int stdout_mode = 0;
 #endif
@@ -420,7 +416,6 @@ co_main (const char *cmd, int argc, char **argv)
   author = date = rev = state = NULL;
   joinflag = NULL;
   expmode = -1;
-  BE (pe) = X_DEFAULT;
   tostdout = false;
   Ttimeflag = false;
 
@@ -433,12 +428,7 @@ co_main (const char *cmd, int argc, char **argv)
 
         case 'r':
         revno:
-          if (*a)
-            {
-              if (rev)
-                PWARN ("redefinition of %s", ks_revno);
-              rev = a;
-            }
+          chk_set_rev (&rev, a);
           break;
 
         case 'f':
@@ -515,10 +505,9 @@ co_main (const char *cmd, int argc, char **argv)
         case 'w':
           if (author)
             redefined ('w');
-          if (*a)
-            author = a;
-          else
-            author = getcaller ();
+          author = (*a)
+            ? a
+            : getcaller ();
           break;
 
         case 'x':
@@ -553,7 +542,7 @@ co_main (const char *cmd, int argc, char **argv)
   /* (End of option processing.)  */
 
   /* Now handle all filenames.  */
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     cleanup (&exitstatus, &neworkptr);
   else if (argc < 1)
     PFATAL ("no input file");
@@ -567,8 +556,9 @@ co_main (const char *cmd, int argc, char **argv)
         ffree ();
 
         if (pairnames
-            (argc, argv, lockflag ? rcswriteopen : rcsreadopen, true,
-             false) <= 0)
+            (argc, argv,
+             lockflag ? rcswriteopen : rcsreadopen,
+             true, false) <= 0)
           continue;
 
         /* ‘REPO (filename)’ contains the name of the RCS file, and
@@ -662,7 +652,9 @@ co_main (const char *cmd, int argc, char **argv)
             /* Check reservations.  */
             changelock = lockflag < 0
               ? rmlock (jstuff.d)
-              : lockflag == 0 ? 0 : addlock_maybe (jstuff.d, selfsame, true);
+              : (lockflag == 0
+                 ? 0
+                 : addlock_maybe (jstuff.d, selfsame, true));
 
             if (changelock < 0
                 || (changelock && !checkaccesslist ())
@@ -703,9 +695,8 @@ co_main (const char *cmd, int argc, char **argv)
             if (changelock && deltas->entry != jstuff.d)
               fro_trundling (true, from);
 
-            if (PROB (donerewrite (changelock, Ttimeflag
-                                   ? repo_stat->st_mtime
-                                   : (time_t) - 1)))
+            if (PROB (donerewrite (changelock,
+                                   file_mtime (Ttimeflag, repo_stat))))
               continue;
 
             if (changelock)
@@ -735,11 +726,13 @@ co_main (const char *cmd, int argc, char **argv)
             mode_t m = WORKMODE (repo_stat->st_mode,
                                  !(kws == kwsub_v
                                    || (lockflag <= 0 && BE (strictly_locking))));
-            time_t t = mtimeflag
-              && newdate ? date2time (newdate) : (time_t) - 1;
+            time_t t = mtimeflag && newdate
+              ? date2time (newdate)
+              : TIME_UNSPECIFIED;
             aflush (neworkptr);
             IGNOREINTS ();
-            r = chnamemod (&neworkptr, neworkname, mani_filename, 1, m, t);
+            r = chnamemod (&neworkptr, neworkname, mani_filename, 1, m,
+                           make_timespec (t, ZERO_NANOSECONDS));
             keepdirtemp (neworkname);
             RESTOREINTS ();
             if (PROB (r))
