@@ -1,6 +1,6 @@
 /* gnu-h-v.c --- GNUish --help and --version handling
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
 
    This file is part of GNU RCS.
 
@@ -41,7 +41,7 @@ nice_getopt (int argc, char **argv, const struct option *longopts)
 
 #define COMMAND_VERSION                                         \
   (" (" PACKAGE_NAME ") " PACKAGE_VERSION "\n"                  \
-   "Copyright (C) 2010-2015 Thien-Thi Nguyen\n"                      \
+   "Copyright (C) 2010-2020 Thien-Thi Nguyen\n"                 \
    "Copyright (C) 1990-1995 Paul Eggert\n"                      \
    "Copyright (C) 1982,1988,1989 Walter F. Tichy, Purdue CS\n"  \
    "License GPLv3+: GNU GPL version 3 or later"                 \
@@ -71,7 +71,6 @@ display_version (struct program const *prog, int flags)
 
 enum hv_option_values
   {
-    hv_unrecognized = 0,
     hv_help,
     hv_version
   };
@@ -91,9 +90,6 @@ check_hv (int argc, char **argv, struct program const *prog)
 
   switch (nice_getopt (argc, argv, ok))
     {
-    case hv_unrecognized:
-      /* Do nothing.  */
-      break;
     case hv_help:
       {
         char usage[128];
