@@ -1,6 +1,6 @@
 /* Parse a string, returning a ‘struct partime’ that describes it.
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1993, 1994, 1995 Paul Eggert
 
    This file is part of GNU RCS.
@@ -48,7 +48,9 @@ struct partime
   long zone;
 };
 
-char const *partime (char const *s, struct partime *t);
-char const *parzone (char const *s, long *zone);
+char const *partime (char const *s, struct partime *t)
+  ALL_NONNULL;
+char const *parzone (char const *s, long *zone)
+  ALL_NONNULL;
 
 /* partime.h ends here */
