@@ -1,6 +1,6 @@
 /* Change RCS file attributes.
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -32,6 +32,8 @@
 #include "b-feph.h"
 #include "b-fro.h"
 
+static const char ks_ws_comma[] = " \n\t,";
+
 struct u_log
 {
   char const *revno;
@@ -71,7 +73,7 @@ struct adminstuff
 
   /* For ‘-sSTATE’ handling.  */
   char const *headstate;
-  bool headstate_changed_p;
+  bool headstate_changed;
   struct link states;
   struct link *tp_state;
 
@@ -95,7 +97,7 @@ struct adminstuff
 static void
 cleanup (int *exitstatus)
 {
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     *exitstatus = exit_failure;
   fro_zclose (&FLOW (from));
   Ozclose (&FLOW (res));
@@ -168,36 +170,28 @@ getchaccess (struct adminstuff *dc,
 }
 
 static void
-getaccessor (struct adminstuff *dc, char *opt, enum changeaccess command)
-/* Get the accessor list of options ‘-e’ and ‘-a’; store in ‘chaccess’.  */
+getaccessor (struct adminstuff *dc, char *argv, enum changeaccess command)
+/* Parse logins from ‘argv’ for ‘command’; call ‘getchaccess’ w/ them.
+   If none specified, either signal error (‘command’ is ‘append’)
+   or call ‘getchaccess’ w/ NULL (‘command’ is ‘erase’).  */
 {
-  register int c;
-  register char *sp;
+  char *s, *save, *who;
+  bool any;
 
-  sp = opt;
-  while ((c = *++sp) == ' ' || c == '\n' || c == '\t' || c == ',')
-    continue;
-  if (c == '\0')
+  for (any = false, s = argv + 2 /* skip "-[ae]" */;
+       (who = strtok_r (s, ks_ws_comma, &save));
+       any = true, s = NULL)
     {
-      if (command == erase && sp - opt == 1)
-        {
-          getchaccess (dc, NULL, command);
-          return;
-        }
-      PERR ("missing login name after option -a or -e");
-      return;
+      checkid (who, 0);
+      getchaccess (dc, who, command);
     }
 
-  while (c != '\0')
-    {
-      size_t len;
-
-      getchaccess (dc, SHSNIP (&len, sp, checkid (sp, ',')), command);
-      sp += len;
-      c = *sp;
-      while (c == ' ' || c == '\n' || c == '\t' || c == ',')
-        c = (*++sp);
-    }
+  if (! any)
+    switch (command)
+      {
+      case append: PERR ("missing login name after option -a");
+      case erase:  getchaccess (dc, NULL, command);
+      }
 }
 
 static void
@@ -253,7 +247,7 @@ getstates (struct adminstuff *dc, char *sp)
   if (c == '\0')
     {
       /* Change state of default branch or ‘REPO (tip)’.  */
-      dc->headstate_changed_p = true;
+      dc->headstate_changed = true;
       dc->headstate = temp;
       return;
     }
@@ -354,9 +348,8 @@ scanlogtext (struct adminstuff *dc,
         }
       else if (nextdelta->pretty_log.string && nextdelta->selector)
         {
-          afputc (SDELIM, to);
-          putstring (to, false, nextdelta->pretty_log, true);
-          afputc ('\n', to);
+          putstring (to, nextdelta->pretty_log, true);
+          newline (to);
         }
       else if (to)
         atat_put (to, log);
@@ -458,7 +451,7 @@ sendmail (char const *Delta, char const *who, bool suppress_mail)
   complain ("Revision %s is already locked by %s.\n", Delta, who);
   if (suppress_mail)
     return true;
-  if (!yesorno (false, "Do you want to break the lock? [ny](n): "))
+  if (!yesorno (false, "Do you want to break the lock"))
     return false;
 
   /* Go ahead with breaking.  */
@@ -595,7 +588,7 @@ removerevs (struct adminstuff *dc)
   struct delta *target, *target2, *temp;
   struct wlink *ls;
   int length;
-  int cmp;
+  bool different;
 
 #define GENREV(x)    gr_revno (x, &ls)
 #define SEARCH(x,l)  searchcutpt (dc, x, l, ls)
@@ -605,14 +598,14 @@ removerevs (struct adminstuff *dc)
   target = GENREV (numrev.string);
   if (!target)
     return false;
-  cmp = cmpnum (target->num, numrev.string);
+  different = !NUM_EQ (target->num, numrev.string);
   length = countnumflds (numrev.string);
 
   if (dc->delrev.code == 0)
     {                           /* -o rev or -o branch */
       if (ODDP (length))
         temp = SEARCH (target->num, length + 1);
-      else if (cmp)
+      else if (different)
         {
           RERR ("Revision %s doesn't exist.", numrev.string);
           return false;
@@ -663,14 +656,13 @@ removerevs (struct adminstuff *dc)
       if (length == 2)
         {
           temp = SEARCH (target->num, 1);
-          if (cmp)
-            dc->cuttail = target;
-          else
-            dc->cuttail = target->ilk;
+          dc->cuttail = different
+            ? target
+            : target->ilk;
         }
       else
         {
-          if (cmp)
+          if (different)
             {
               dc->cuthead = target;
               if (!(temp = target->ilk))
@@ -707,12 +699,12 @@ removerevs (struct adminstuff *dc)
     {                           /* delete revisions on branches */
       if (NUM_GT (target->num, target2->num))
         {
-          cmp = cmpnum (target2->num, numrev.string);
+          different = !NUM_EQ (target2->num, numrev.string);
           temp = target;
           target = target2;
           target2 = temp;
         }
-      if (cmp)
+      if (different)
         {
           if (NUM_EQ (target->num, target2->num))
             {
@@ -736,8 +728,8 @@ removerevs (struct adminstuff *dc)
           target2 = temp;
         }
       else
-        cmp = cmpnum (target2->num, numrev.string);
-      if (cmp)
+        different = !NUM_EQ (target2->num, numrev.string);
+      if (different)
         {
           if (NUM_EQ (target->num, target2->num))
             {
@@ -1075,9 +1067,7 @@ buildtree (struct adminstuff *dc)
     {
       if (!dc->cuttail && !BE (quiet))
         {
-          if (!yesorno
-              (false,
-               "Do you really want to delete all revisions? [ny](n): "))
+          if (!yesorno (false, "Do you really want to delete all revisions"))
             {
               RERR ("No revision deleted");
               Delta = dc->delstrt;
@@ -1126,7 +1116,6 @@ rcs_main (const char *cmd, int argc, char **argv)
   boxrm.next = dc.byelocks;
   tprm = &boxrm;
   expmode = -1;
-  BE (pe) = X_DEFAULT;
   initflag = textflag = false;
   strict_selected = false;
   Ttimeflag = false;
@@ -1165,7 +1154,7 @@ rcs_main (const char *cmd, int argc, char **argv)
 
         case 'a':
           /* Add new accessor.  */
-          getaccessor (&dc, *argv + 1, append);
+          getaccessor (&dc, *argv, append);
           break;
 
         case 'A':
@@ -1186,7 +1175,7 @@ rcs_main (const char *cmd, int argc, char **argv)
 
         case 'e':
           /* Remove accessors.  */
-          getaccessor (&dc, *argv + 1, erase);
+          getaccessor (&dc, *argv, erase);
           break;
 
         case 'l':
@@ -1332,7 +1321,7 @@ rcs_main (const char *cmd, int argc, char **argv)
   /* (End processing of options.)  */
 
   /* Now handle all filenames.  */
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     cleanup (&dc.rv);
   else if (argc < 1)
     PFATAL ("no input file");
@@ -1444,7 +1433,7 @@ rcs_main (const char *cmd, int argc, char **argv)
         changed |= domessages (&dc);
 
         /* Update state attribution.  */
-        if (dc.headstate_changed_p)
+        if (dc.headstate_changed)
           {
             /* Change state of default branch or head.  */
             if (!defbr)
@@ -1479,7 +1468,7 @@ rcs_main (const char *cmd, int argc, char **argv)
             keepRCStime = false;
           }
 
-        if (FLOW (erroneousp))
+        if (FLOW (erroneous))
           continue;
 
         putadmin ();
@@ -1534,9 +1523,8 @@ rcs_main (const char *cmd, int argc, char **argv)
             repo_stat->st_nlink = 0;
             keepRCStime = false;
           }
-        if (PROB (donerewrite (changed, keepRCStime
-                               ? repo_stat->st_mtime
-                               : (time_t) - 1)))
+        if (PROB (donerewrite (changed,
+                               file_mtime (keepRCStime, repo_stat))))
           break;
 
         diagnose ("done");
