@@ -1,6 +1,6 @@
 /* Clean up working files.
 
-   Copyright (C) 2010-2014 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1991, 1992, 1993, 1994, 1995 Paul Eggert
 
    This file is part of GNU RCS.
@@ -37,7 +37,7 @@
 static void
 cleanup (int *exitstatus, struct fro **workptr)
 {
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     *exitstatus = exit_failure;
   fro_zclose (&FLOW (from));
   fro_zclose (workptr);
@@ -63,41 +63,44 @@ unlock (struct delta *delta)
 }
 
 static int
-get_directory (char const *dirname, char ***aargv)
-/* Put a vector of all DIRNAME's directory entries names into *AARGV.
-   Ignore names of RCS files.
-   Return the number of entries found.  Terminate the vector with 0.
-   Allocate the storage for the vector and entry names.
-   Do not sort the names.  Do not include '.' and '..'.  */
+valid_filename_p (const struct dirent *ent)
+/* Filter out ".", "..", and comma-v filenames.
+   This func is a predicate for ‘get_cwd_filenames’, q.v.  */
 {
-  DIR *d;
-  struct dirent *e;
-  struct divvy *justme = make_space ("justme");
-  size_t i, entries = 0;
-  struct wlink head, *tp;
+  const char *en = ent->d_name;
 
-  if (!(d = opendir (dirname)))
-    fatal_sys (dirname);
-  head.next = NULL;
-  tp = &head;
-  while ((errno = 0, e = readdir (d)))
+  if (en[0] == '.' && (!en[1] || (en[1] == '.' && !en[2])))
+    return 0;
+
+  if (rcssuffix (en))
+    return 0;
+
+  return 1;
+}
+
+static int
+get_cwd_filenames (char ***aargv)
+/* Put a vector of unsorted cwd's filenames into *AARGV.
+   Ignore names of RCS files, as well as "." and "..".
+   Allocate storage for the vector and entry names.
+   Return the number of entries found.  */
+{
+  char dot[] = ".";
+  struct dirent **names;
+  int count, i;
+
+  if (PROB (count = scandir (dot, &names, valid_filename_p, NULL)))
+    fatal_sys (dot);
+
+  *aargv = pointer_array (PLEXUS, count);
+  for (i = count; i--;)
     {
-      char const *en = e->d_name;
-
-      if (en[0] == '.' && (!en[1] || (en[1] == '.' && !en[2])))
-        continue;
-      if (rcssuffix (en))
-        continue;
-      tp = wextend (tp, str_save (en), justme);
-      entries++;
+      (*aargv)[i] = str_save (names[i]->d_name);
+      free (names[i]);
     }
-  if (errno || PROB (closedir (d)))
-    fatal_sys (dirname);
-  *aargv = pointer_array (PLEXUS, entries);
-  for (tp = head.next, i = 0; i < entries; tp = tp->next, i++)
-    (*aargv)[i] = tp->entry;
-  close_space (justme);
-  return entries;
+  free (names);
+
+  return count;
 }
 
 DECLARE_PROGRAM (rcsclean, BOG_FULL);
@@ -122,7 +125,6 @@ rcsclean_main (const char *cmd, int argc, char **argv)
 
   expmode = -1;
   rev = NULL;
-  BE (pe) = X_DEFAULT;
   perform = true;
   unlockflag = false;
   Ttimeflag = false;
@@ -133,7 +135,7 @@ rcsclean_main (const char *cmd, int argc, char **argv)
     {
       if (--argc < 1)
         {
-          argc = get_directory (".", &newargv);
+          argc = get_cwd_filenames (&newargv);
           argv = newargv;
           break;
         }
@@ -158,12 +160,7 @@ rcsclean_main (const char *cmd, int argc, char **argv)
           /* fall into */
         case 'r':
         handle_revision:
-          if (*a)
-            {
-              if (rev)
-                PWARN ("redefinition of %s", ks_revno);
-              rev = a;
-            }
+          chk_set_rev (&rev, a);
           break;
 
         case 'T':
@@ -196,7 +193,7 @@ rcsclean_main (const char *cmd, int argc, char **argv)
 
   dounlock = perform & unlockflag;
 
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     cleanup (&exitstatus, &workptr);
   else
     for (; 0 < argc; cleanup (&exitstatus, &workptr), ++argv, --argc)
@@ -285,9 +282,7 @@ rcsclean_main (const char *cmd, int argc, char **argv)
             SAME_AFTER (from, delta->text);
             if (deltas->entry != delta)
               fro_trundling (true, from);
-            if (PROB (donerewrite (true, Ttimeflag
-                                   ? repo_stat->st_mtime
-                                   : (time_t) - 1)))
+            if (PROB (donerewrite (true, file_mtime (Ttimeflag, repo_stat))))
               continue;
           }
 
