@@ -1,6 +1,6 @@
 /* Generate RCS revisions.
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -133,7 +133,9 @@ buildrevision (struct wlink const *deltas, struct delta *target,
     {
       /* Only latest revision to generate.  */
       openfcopy (outfile);
-      scandeltatext (es, &ls, target, expandflag ? expand : copy, true);
+      scandeltatext (es, &ls, target,
+                     expandflag ? expand : copy,
+                     true);
     }
   else
     {
@@ -151,8 +153,12 @@ buildrevision (struct wlink const *deltas, struct delta *target,
           /* First, get to beginning of file.  */
           finishedit (es, NULL, outfile, false);
         }
-      scandeltatext (es, &ls, target, expandflag ? edit_expand : edit, true);
-      finishedit (es, expandflag ? target : NULL, outfile, true);
+      scandeltatext (es, &ls, target,
+                     expandflag ? edit_expand : edit,
+                     true);
+      finishedit (es,
+                  expandflag ? target : NULL,
+                  outfile, true);
     }
   unmake_editstuff (es);
   if (outfile)
@@ -217,10 +223,15 @@ yesorno (bool default_answer, char const *question, ...)
 
   if (!BE (quiet) && ttystdin ())
     {
+      char *ans = default_answer
+        ? "yn"
+        : "ny";
+
       oflush ();
       va_start (args, question);
       vcomplain (question, args);
       va_end (args);
+      complain ("? [%s](%c): ", ans, ans[0]);
       r = c = getcstdin ();
       while (c != '\n' && !feof (stdin))
         c = getcstdin ();
@@ -269,7 +280,7 @@ putdesc (struct cbuf *cb, bool textflag, char *textfile)
     {
       FLOW (to) = NULL;
       /* Get new description.  */
-      aprintf (frew, "\n\n%s\n%c", TINYKS (desc), SDELIM);
+      aprintf (frew, "\n\n%s\n", TINYKS (desc));
       if (!textfile)
         *cb = getsstdin ("t-", "description",
                          "NOTE: This is NOT the log message!\n");
@@ -300,8 +311,8 @@ putdesc (struct cbuf *cb, bool textflag, char *textfile)
             }
           *cb = cleanlogmsg (p, s);
         }
-      putstring (frew, false, *cb, true);
-      aputc ('\n', frew);
+      putstring (frew, *cb, true);
+      newline (frew);
     }
 }
 
@@ -312,7 +323,7 @@ getsstdin (char const *option, char const *name, char const *note)
   register char *p;
   register bool tty = ttystdin ();
   size_t len, column = 0;
-  bool dot_in_first_column_p = false, discardp = false;
+  bool dot_in_first_column = false, discard = false;
 
 #define prompt  complain
   if (tty)
@@ -325,12 +336,12 @@ getsstdin (char const *option, char const *name, char const *note)
   while (c = getcstdin (), !feof (stdin))
     {
       if (!column)
-        dot_in_first_column_p = ('.' == c);
+        dot_in_first_column = ('.' == c);
       if (c == '\n')
         {
-          if (1 == column && dot_in_first_column_p)
+          if (1 == column && dot_in_first_column)
             {
-              discardp = true;
+              discard = true;
               break;
             }
           else if (tty)
@@ -343,7 +354,7 @@ getsstdin (char const *option, char const *name, char const *note)
     }
   p = finish_string (PLEXUS, &len);
 #undef prompt
-  return cleanlogmsg (p, len - (discardp ? 1 : 0));
+  return cleanlogmsg (p, len - (discard ? 1 : 0));
 }
 
 void
@@ -400,11 +411,15 @@ putadmin (void)
         fatal_sys (REPO (filename));
     }
 
-  aprintf (fout, "%s\t%s%s", TINYKS (head), tip ? tip->num : "", semi_lf);
+  aprintf (fout, "%s\t%s%s", TINYKS (head),
+           tip ? tip->num : "",
+           semi_lf);
   if (defbr && VERSION (4) <= BE (version))
     aprintf (fout, "%s\t%s%s", TINYKS (branch), defbr, semi_lf);
   aputs (TINYKS (access), fout);
-  for (struct link *ls = r ? GROK (access) : NULL; ls; ls = ls->next)
+  for (struct link *ls = r ? GROK (access) : NULL;
+       ls;
+       ls = ls->next)
     aprintf (fout, "\n\t%s", (char const *) ls->entry);
   SEMI_LF ();
   aprintf (fout, "%s", TINYKS (symbols));
@@ -423,7 +438,7 @@ putadmin (void)
   if (REPO (log_lead).size)
     {
       aprintf (fout, "%s\t", TINYKS (comment));
-      putstring (fout, true, REPO (log_lead), false); SEMI_LF ();
+      putstring (fout, REPO (log_lead), false); SEMI_LF ();
     }
   if (kws != kwsub_kv)
     aprintf (fout, "%s\t%c%s%c%s",
@@ -441,7 +456,8 @@ putdelta (register struct delta const *node, register FILE *fout)
 
   aprintf (fout, "\n%s\n%s\t%s;\t%s %s;\t%s %s%s%s",
            node->num, TINYKS (date), node->date, TINYKS (author), node->author,
-           TINYKS (state), node->state ? node->state : "",
+           TINYKS (state),
+           node->state ? node->state : "",
            semi_lf, TINYKS (branches));
   for (struct wlink *ls = node->branches; ls; ls = ls->next)
     {
@@ -451,7 +467,8 @@ putdelta (register struct delta const *node, register FILE *fout)
     }
   SEMI_LF ();
 
-  aprintf (fout, "%s\t%s", TINYKS (next), node->ilk ? node->ilk->num : "");
+  aprintf (fout, "%s\t%s", TINYKS (next),
+           node->ilk ? node->ilk->num : "");
   SEMI_LF ();
   if (node->commitid)
     aprintf (fout, "%s\t%s%s", TINYKS (commitid), node->commitid, semi_lf);
@@ -461,16 +478,23 @@ void
 puttree (struct delta const *root, register FILE *fout)
 /* Output the delta tree with base ‘root’ in preorder to ‘fout’.  */
 {
-  if (!root)
-    return;
+  while (root)
+    {
+      if (root->selector)
+        putdelta (root, fout);
 
-  if (root->selector)
-    putdelta (root, fout);
-
-  puttree (root->ilk, fout);
-
-  for (struct wlink *ls = root->branches; ls; ls = ls->next)
-    puttree (ls->entry, fout);
+      struct wlink *ls = root->branches;
+      if (! ls)
+        root = root->ilk;
+      else
+        {
+          /* FIXME: This recurses deeply in the worst case.  */
+          puttree (root->ilk, fout);
+          for (; ls->next; ls = ls->next)
+            puttree (ls->entry, fout);
+          root = ls->entry;
+        }
+    }
 }
 
 bool
@@ -494,27 +518,39 @@ putdtext (struct delta const *delta, char const *srcname,
   return true;
 }
 
+static void
+put_SDELIM (FILE *out)
+{
+  aputc (SDELIM, out);
+}
+
 void
-putstring (register FILE *out, bool delim, struct cbuf s, bool log)
-/* Output to ‘out’ one ‘SDELIM’ if ‘delim’, then the string ‘s’ with
-   ‘SDELIM’s doubled.  If ‘log’ is set then ‘s’ is a log string; append
-   a newline if ‘s’ is nonempty.  */
+putstring (register FILE *out, struct cbuf s, bool log)
+/* Output to ‘out’ one ‘SDELIM’, then string ‘s’ with ‘SDELIM’s doubled.
+   If ‘log’ is set then ‘s’ is a log string; append a newline if ‘s’ is
+   nonempty.  Finally, output a terminating ‘SDELIM’.  */
 {
   register char const *sp;
-  register size_t ss;
+  char const *delim;
+  register size_t ss, span;
 
-  if (delim)
-    aputc (SDELIM, out);
-  sp = s.string;
-  for (ss = s.size; ss; --ss)
+  put_SDELIM (out);
+  for (sp = s.string, ss = s.size;
+       ss;
+       sp += span, ss -= span)
     {
-      if (*sp == SDELIM)
-        aputc (SDELIM, out);
-      aputc (*sp++, out);
+      delim = memchr (sp, SDELIM, ss);
+      span = delim
+        ? (size_t) (delim - sp) + 1U
+        : ss;
+
+      awrite (sp, span, out);
+      if (delim)
+        put_SDELIM (out);
     }
   if (s.size && log)
-    aputc ('\n', out);
-  aputc (SDELIM, out);
+    newline (out);
+  put_SDELIM (out);
 }
 
 void
@@ -532,8 +568,8 @@ putdftext (struct delta const *delta, struct fro *finfile,
   aprintf (fout, "\n\n%s\n%s\n", delta->num, TINYKS (log));
 
   /* Put log.  */
-  putstring (fout, true, delta->pretty_log, true);
-  aputc ('\n', fout);
+  putstring (fout, delta->pretty_log, true);
+  newline (fout);
   /* Put text.  */
   aprintf (fout, "%s\n%c", TINYKS (text), SDELIM);
 
@@ -546,7 +582,7 @@ putdftext (struct delta const *delta, struct fro *finfile,
           GETCHAR_OR (c, fin, goto done);
           if (c == SDELIM)
             /* Double up ‘SDELIM’.  */
-            aputc (SDELIM, fout);
+            put_SDELIM (fout);
           aputc (c, fout);
         }
     done:
@@ -568,7 +604,7 @@ putdftext (struct delta const *delta, struct fro *finfile,
                                 unexpected_EOF ();
                               });
                   if (c == SDELIM)
-                    aputc (SDELIM, fout);
+                    put_SDELIM (fout);
                   aputc (c, fout);
                 }
               while (c != '\n');
