@@ -1,6 +1,6 @@
 /* Extract RCS keyword string values from working files.
 
-   Copyright (C) 2010-2014 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Copyright (C) 1982, 1988, 1989 Walter Tichy
 
@@ -29,9 +29,9 @@
 #include <ctype.h>
 
 static char *
-sorry (bool savep, char const *msg)
+sorry (bool save, char const *msg)
 {
-  if (savep)
+  if (save)
     {
       char *partial;
       size_t len;
@@ -45,13 +45,13 @@ sorry (bool savep, char const *msg)
 }
 
 static char *
-badly_terminated (bool savep)
+badly_terminated (bool save)
 {
-  return sorry (savep, "badly terminated keyword value");
+  return sorry (save, "badly terminated keyword value");
 }
 
 static char *
-get0val (int c, register struct fro *fp, bool savep, bool optional)
+get0val (int c, register struct fro *fp, bool save, bool optional)
 /* Read a keyword value from ‘c + fp’, perhaps ‘optional’ly.
    Same as ‘getval’, except ‘c’ is the lookahead character.  */
 {
@@ -66,13 +66,13 @@ get0val (int c, register struct fro *fp, bool savep, bool optional)
         {
         default:
           got1 = true;
-          if (savep)
+          if (save)
             accumulate_byte (SINGLE, c);
           break;
 
         case ' ':
         case '\t':
-          if (savep)
+          if (save)
             {
               val = finish_string (SINGLE, &len);
 #ifdef KEEPTEST
@@ -99,9 +99,9 @@ get0val (int c, register struct fro *fp, bool savep, bool optional)
           /* fall into */
         case '\n':
         case '\0':
-          return badly_terminated (savep);
+          return badly_terminated (save);
         }
-      GETCHAR_OR (c, fp, return badly_terminated (savep));
+      GETCHAR_OR (c, fp, return badly_terminated (save));
     }
 }
 
@@ -116,7 +116,7 @@ keepid (int c, struct fro *fp)
   if (!(maybe = get0val (c, fp, true, false)))
     return NULL;
   checksid (maybe);
-  if (FLOW (erroneousp))
+  if (FLOW (erroneous))
     {
       brush_off (SINGLE, maybe);
       maybe = NULL;
@@ -125,14 +125,14 @@ keepid (int c, struct fro *fp)
 }
 
 static char *
-getval (register struct fro *fp, bool savep, bool optional)
+getval (register struct fro *fp, bool save, bool optional)
 /* Read a keyword value from ‘fp’; return it if found, else NULL.
    Do not report an error if ‘optional’ is set and ‘kdelim’ is found instead.  */
 {
   int c;
 
-  GETCHAR_OR (c, fp, return badly_terminated (savep));
-  return get0val (c, fp, savep, optional);
+  GETCHAR_OR (c, fp, return badly_terminated (save));
+  return get0val (c, fp, save, optional);
 }
 
 static int
@@ -318,7 +318,7 @@ getoldkeys (register struct fro *fp)
               /* Skip either ``who'' (new form) or ``Locker: who'' (old).  */
               if (getval (fp, false, true) && getval (fp, false, true))
                 c = 0;
-              else if (FLOW (erroneousp))
+              else if (FLOW (erroneous))
                 goto badness;
               else
                 c = KDELIM;
