@@ -1,6 +1,6 @@
 /* btdt.c --- been there done that
 
-   Copyright (C) 2010-2015 Thien-Thi Nguyen
+   Copyright (C) 2010-2020 Thien-Thi Nguyen
 
    This file is part of GNU RCS.
 
@@ -21,6 +21,8 @@
 #include "base.h"
 #include <string.h>
 #include <stdlib.h>
+#include "stat-time.h"
+#include "timespec.h"
 #include "b-complain.h"
 #include "b-divvy.h"
 #include "b-esds.h"
@@ -135,6 +137,52 @@ xorlf_do_it (int argc, char *argv[VLA_ELEMS (argc)] RCS_UNUSED)
   return EXIT_SUCCESS;
 }
 
+
+/* mtimecmp */
+
+/* Consider mtime of both FILE1 and FILE2 as M1 and M2.
+   If M1 < M2, display "-1" to stdout.
+   If M1 = M2, display "0" to stdout.
+   If M1 > M2, display "1" to stdout.  */
+
+char const mtimecmp_usage[] =
+  "FILE1 FILE2";
+
+struct timespec
+mtimecmp_grok (char const *filename)
+{
+  struct stat st;
+
+  if (PROB (stat (filename, &st)))
+    {
+      fprintf (stderr, "mtimecmp: could not stat %s\n",
+               filename);
+      _Exit (EXIT_FAILURE);
+    }
+
+  return get_stat_mtime (&st);
+}
+
+int
+mtimecmp_do_it (int argc, char *argv[VLA_ELEMS (argc)])
+{
+  struct timespec m1, m2;
+  long int sign;
+
+  if (3 > argc)
+    bad_args (argv[0]);
+
+  m1 = mtimecmp_grok (argv[1]);
+  m2 = mtimecmp_grok (argv[2]);
+
+  sign = timespec_cmp (m1, m2);
+  if (0 > sign) sign = -1;
+  if (0 < sign) sign =  1;
+
+  printf ("%ld\n", sign);
+  return EXIT_SUCCESS;
+}
+
 
 typedef int (main_t) (int argc, char *argv[VLA_ELEMS (argc)]);
 
@@ -143,7 +191,7 @@ struct yeah
   char const *component;
   char const *usage;
   main_t *whatever;
-  bool scramp;
+  bool scram;
 };
 
 #define YEAH(comp,out)  { #comp, comp ## _usage, comp ## _do_it, out }
@@ -153,6 +201,7 @@ struct yeah yeah[] =
     YEAH (getoldkeys,   true),
     YEAH (grok,         true),
     YEAH (xorlf,        true),
+    YEAH (mtimecmp,     true),
   };
 
 #define NYEAH  (sizeof (yeah) / sizeof (struct yeah))
@@ -165,7 +214,7 @@ main (int argc, char *argv[VLA_ELEMS (argc)])
   if (STR_SAME ("--version", argv[1]))
     {
       printf ("btdt (%s) %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-      printf ("Copyright (C) 2010-2015 Thien-Thi Nguyen\n");
+      printf ("Copyright (C) 2010-2020 Thien-Thi Nguyen\n");
       printf ("License GPLv3+; GNU GPL version 3 or later"
               " <http://gnu.org/licenses/gpl.html>\n\n");
       argv[1] = "--help";
@@ -189,7 +238,7 @@ main (int argc, char *argv[VLA_ELEMS (argc)])
           {
             .invoke = me,
             .name  = argv[1],
-            .tyag = (yeah[i].scramp
+            .tyag = (yeah[i].scram
                      ? TYAG_IMMEDIATE
                      : BOG_ZONK)
           };
